import 'package:data_connection_checker/data_connection_checker.dart';

import '../../core/core.dart';
import '../../m4e_sdk_initialize.dart';
import '../../models/models.dart';
import '../../m4e_forms/m4e_forms.dart';
import '../api/apis.dart';

/// Exposes SDK Wallet Api methods
///
/// Accessing the instantiated `m4eWallet` methods
/// is done by:
///
/// Instantiating as a regular object
///
/// ```
/// final _m4eWallet = M4eWallet();
///
/// await _m4eWallet.getUserWallets();
/// ```
///
class M4eWalletService {
  final WalletApi _walletApi;
  final PermissionApi _permissionApi;
  final DataConnectionChecker _connectionChecker;

  M4eWalletService({
    WalletApi walletApi,
    PermissionApi permissionApi,
    DataConnectionChecker connectionChecker,
  })  : _walletApi = walletApi ?? m4eSDK<WalletApi>(),
        _permissionApi = permissionApi ?? m4eSDK<PermissionApi>(),
        _connectionChecker =
            connectionChecker ?? m4eSDK<DataConnectionChecker>();

  /// Returns current authenticated m4e user `Wallets` from m4e when no [EntityWalletAccess] is provided
  ///
  /// Returns m4e user `Wallets` with provided `M4eUniqueId` and [EntityWalletAccess] permissions from m4e
  ///
  /// Example:
  ///
  /// ```
  /// final service = M4eWalletService(); // instantiate M4eWalletService
  ///
  /// final currentUserWallets = service.getWallets(97fdebd0-7424-474e-a17c-67be85cc5638); // returns current authenticated m4e user wallets
  /// ```
  ///
  /// ```
  /// final otherUserWallets = service.getWallets(97fdebd0-7424-474e-a17c-67be85cc5638, ['SEE', 'VIEW']);  // returns m4e-user with [M4eUniqueId] wallets which the current authenticated user has a `SEE` and 'VIEW' permission on
  /// ```
  ///
  /// Throws [M4eServerException] if request fails.
  ///
  /// Throws `[ServerException]` with `NO_INTERNET_CONNECTION` error
  /// code when device is not connected to the internet
  ///
  Future<M4eWalletList> getWallets(M4eUniqueId userId,
      [List<EntityWalletAccess> permissions]) async {
    if ((await _connectionChecker.hasConnection) ?? false) {
      try {
        // sets owner permissions if user-permission is null
        final _permissions = permissions ??
            <M4eAccessPermissionSet>[
              EntityWalletAccess.see,
              EntityWalletAccess.view,
              EntityWalletAccess.pay,
              EntityWalletAccess.grant,
              EntityWalletAccess.alter,
              EntityWalletAccess.create,
              EntityWalletAccess.destroy,
            ];

        final _wallets = await _walletApi.getUserWallets(userId, _permissions);

        return _wallets;
      } catch (e) {
        rethrow;
      }
    }

    throw M4eExceptionMessages.kNoInternetConnectionException;
  }

  /// Creates an m4e user-wallet and returns created wallet [M4eUniqueId]
  ///
  /// Creates owner-permission on wallet
  ///
  /// Throws [M4eServerException] if request fails.
  ///
  /// Throws `[ServerException]` with `NO_INTERNET_CONNECTION` error
  /// code device is not connected to the internet
  ///
  Future<M4eUniqueId> createWallet(
      M4eUniqueId userId, M4eWalletForm m4eWalletForm) async {
    if ((await _connectionChecker.hasConnection) ?? false) {
      try {
        final _walletId = await _walletApi.createWallet(userId, m4eWalletForm);

        // create EntityWalletAccess permission form
        final _permissionForm = M4ePermissionForm(
          subId: userId,
          objId: _walletId,
          type: EntityWalletAccess.TYPE,
          encoding: <M4eAccessPermissionSet>[
            EntityWalletAccess.see,
            EntityWalletAccess.view,
            EntityWalletAccess.pay,
            EntityWalletAccess.grant,
            EntityWalletAccess.alter,
            EntityWalletAccess.create,
            EntityWalletAccess.destroy,
          ],
        );

        // set owner permissions on wallet
        await _permissionApi.setPermission(_permissionForm);

        return _walletId;
      } catch (e) {
        rethrow;
      }
    }

    throw M4eExceptionMessages.kNoInternetConnectionException;
  }
}
